function [imageRecon] = subsampleSpectrum2d(inputImage,M,scenario)
% [imageRecon] = subsampleSpectrum2d(inputImage,M,scenario)
%
% Reconstruct a 2d image from undersampled frequency spectrum.
%
% Input:
%   inputImage = original image
%   M = number of frequency-domain samples drawn randomly
%   scenario = 1: random sampling without replacement
%   scenario = 2: random sampling with replacement
%
% Output: 
%   imageRecon = reconstructed signal in time domain based on
%                 incomplete frequency-domain samples
%
% Random Spectral Sampling Toolbox - Roel Snieder and Michael B. Wakin
% Version 1.0 - March 23, 2022
% See README.txt

% Take the 2D DFT of the original image
[Nx,Ny] = size(inputImage);
N = Nx*Ny;
idealSpectrum = fft2(inputImage);

% Define the frequency-domain locations to keep spectrum samples
switch scenario
    case 1
        % uniform random sampling without replacement
        sampleProbabilities = (1/N)*ones(N,1);
        sampleLocs = datasample(1:N,M,'Weights',sampleProbabilities,'Replace',false);
    case 2
        % uniform random sampling with replacement
        sampleProbabilities = (1/N)*ones(N,1);
        sampleLocs = datasample(1:N,M,'Weights',sampleProbabilities,'Replace',true);
end
spectralSamples = idealSpectrum(sampleLocs);

% If sampling with replacement, some samples may be "kept" more than once. 
% Define subsampled spectrum with each sample scaled by its multiplicity 
% in spectralSamples.
subsampledSpectrum = zeros(Nx,Ny);
for ii = 1:M
    subsampledSpectrum(sampleLocs(ii)) = subsampledSpectrum(sampleLocs(ii)) + spectralSamples(ii);
end

% Reconstruct from the spectral samples
imageRecon = (N/M)*ifft2(subsampledSpectrum);
